/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package buddhabrot;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.Date;
import javax.imageio.ImageIO;

/**
 *
 * @author zerjillo
 */
public class Buddhabrot {

    int[][] image;
    int pw;
    int ph;
    double scale;
    int maxIter;
    double pixelWidth;
    double minX;
    double minY;
    double maxX;
    double maxY;

    public Buddhabrot(int pw, int ph, double scale, int maxIter, String fileName, String format, long maxTotalCount) {
        this.pw = pw;
        this.ph = ph;
        this.scale = scale;
        this.maxIter = maxIter;
        image = new int[pw][ph];

        pixelWidth = scale / pw;
        minX = (-pw / 1.45) * pixelWidth;
        minY = (-ph / 2) * pixelWidth;

        System.out.println(minX + ", " + minY + " - " + pixelWidth);
        double width = pw * pixelWidth;
        double height = ph * pixelWidth;

        maxX = minX + width;
        maxY = minY + height;

        System.out.println(width + ", " + height);

        long initTime = System.currentTimeMillis();

        long lastTime = System.currentTimeMillis() - 50 * 1000;

        for (long i = 0 ; i < 100000000000l ; i++) {
            if (i % 10000 == 0) {
                long time = System.currentTimeMillis();

                if (time - lastTime > 1 * 10 * 1000) {
                    lastTime = time;
                    System.out.println("MAX ITERS: " + maxIter);
                    System.out.println(new Date().toString());
                    System.out.println(i);
                    //   guardaImagen(fileName, format);

                    long totalCount = Utils.computeTotalCount(image);
                    System.out.println("TOTAL: " + totalCount);

                    if (totalCount > maxTotalCount) {
                        System.out.println("Grabando");
                        Utils.saveImageAsArray(fileName, image);
                        Utils.guardaImagen(fileName, format, image);
                        System.out.println("Saliendo");
                        long endTime = System.currentTimeMillis();

                        try {
                            PrintWriter out = new PrintWriter(fileName + ".txt");
                            out.println("Computing time");
                            out.println((endTime - initTime) + "");
                            out.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                        return;
                    }
                }
            }
            double rx = Math.random() * width + minX;
            double ry = Math.random() * height + minY;
            //     System.out.println(rx + "++++ " + ry);
//double rx = Math.random();
//double ry = Math.random();
            if (pointEscapesMandelbrot(rx, ry)) {
                iterateBuddhabrot(rx, ry);
            } else {

                //   System.out.println("*");
            }
        }
        Utils.guardaImagen(fileName, format, image);

    }


 

    private void iterateBuddhabrot(double re, double im) {
        double x = 0;
        double y = 0;

        for (int i = 0 ; i < maxIter ; i++) {
            double nx = x * x - y * y;
            double ny = 2 * x * y;

            x = nx + re;
            y = ny + im;
            if ((x == Double.NaN) || (y == Double.NaN) || (x == Double.POSITIVE_INFINITY) || (y == Double.POSITIVE_INFINITY) || (x == Double.NEGATIVE_INFINITY) || (y == Double.NEGATIVE_INFINITY)) {
                return;
            }
            if ((x * x + y * y) > 100) {
                return;
            }

            if ((x >= minX) && (x < maxX) && (y >= minY) && (y < maxY)) {
                int px = (int) ((x - minX) / pixelWidth);
                int py = (int) ((y - minY) / pixelWidth);

                //    System.out.println(x  + " --- " + y);
                //    System.out.println(px  + " +++ " + py);
                //  if ((px >= 0) && (px < pw) && (py >= 0) && (py < ph)) {
                image[px][py]++;

                py = ph - 1 - py;

                image[px][py]++;
                //  }

            }

        }
    }

    

    private boolean pointEscapesMandelbrot(double re, double im) {
        double x = 0;
        double y = 0;

        double im2 = im * im;
        double rePlus1 = re+1;
        if ((rePlus1) * (rePlus1) + im2 < 0.0625) {
            return false;
        }

        double reMinus025 = re - 0.25;
        double q = (reMinus025) * (reMinus025) + im2;
        if ((q * (q + reMinus025)) < 0.25 * im2) {
            return false;
        }
        
        for (int i = 0 ; i < maxIter ; i++) {
            double nx = x * x - y * y;
            double ny = 2 * x * y;

            x = nx + re;
            y = ny + im;

            //   System.out.println(x + " - " + y);
            if ((x * x + y * y) > 4) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        //int alto = 9933;
        //int ancho = 14043;
        //int alto = 1080;
        //int ancho = 1920;
        int alto = 2160;
        int ancho = 4096;

        String format = "jpg";
        long maxTotalCount = 3000000000l;

        /*        for (int iters = 566+64+32 ; iters < 10000 ; iters+=64) {
         Buddhabrot bb = new Buddhabrot(ancho, alto, 3.3, iters, "fractal" + String.format("%05d", iters) + "", format, maxTotalCount);
         }*/
        int iters = 0;
         for (int i = 0 ; i < 1000 ; i++) {
         int inc = (int) Math.pow(1.15, Math.sqrt(i));

         iters += inc;

         System.out.println(iters);

         File nf = new File(iters + ".touch");

         if (!nf.exists()) {
         try {
         new FileOutputStream(nf).close();
         } catch (IOException e) {
         }
         Buddhabrot bb = new Buddhabrot(ancho, alto, 3.3, iters, "fractal" + String.format("%05d", iters) + "", format, maxTotalCount);
         } else {
                
         }
         }

         System.exit(0);
      
        /*int iters = 500;
        Buddhabrot bb = new Buddhabrot(ancho, alto, 3.3, iters, "fractal" + String.format("%05d", iters) + "", format, maxTotalCount);
*/
    }

}
