/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package buddhabrot;

/**
 *
 * @author zerjillo
 */
public class BuddhabrotComputingThread extends Thread {

    boolean exit;
    double width;
    double height;
    double minX;
    double maxX;
    double minY;
    double maxY;
    int maxIter;
    double pixelWidth;
    int[][] image;
    boolean paused;
    
    public BuddhabrotComputingThread(double minX, double minY, int maxIter, double pixelWidth, int[][] image) {
        exit = false;

        this.minX = minX;
        this.minY = minY;
                this.image = image;
                int imageWidth = image.length;
                int imageHeight = image[0].length;
                
                this.maxX = minX + pixelWidth * imageWidth;
                this.maxY = minY + pixelWidth * imageHeight;
                
                System.out.println("(" + minX + ", " + minY + ") -- (" + maxX + ", " + maxY + ")");
                
        this.width = maxX - minX;
        this.height = maxY - minY;
        this.maxIter = maxIter;
        this.pixelWidth = pixelWidth;

        paused = false;
    }

    public void exit() {
        exit = true;
    }
    
    public void setPaused(boolean paused) {
        this.paused = paused;
    }
    
    public void run() {
        while (!exit) {
            while (paused) {
                Utils.delay(30);
            }
            
      //      double rx = Math.random() * width + minX;
      //      double ry = Math.random() * height + minY;

            double rx = Math.random() * 6 - 3;
            double ry = Math.random() * 6 - 3;            
            
            if (pointEscapesMandelbrot(rx, ry)) {
                iterateBuddhabrot(rx, ry);
            }
        }
    }
    
    
    private boolean pointEscapesMandelbrot(double re, double im) {
        double x = 0;
        double y = 0;

        double im2 = im * im;
        double rePlus1 = re + 1;
        if ((rePlus1) * (rePlus1) + im2 < 0.0625) {
            return false;
        }

        double reMinus025 = re - 0.25;
        double q = (reMinus025) * (reMinus025) + im2;
        if ((q * (q + reMinus025)) < 0.25 * im2) {
            return false;
        }

        for (int i = 0 ; i < maxIter ; i++) {
            double nx = x * x - y * y;
            double ny = 2 * x * y;

            x = nx + re;
            y = ny + im;

            if ((x * x + y * y) > 4) {
                return true;
            }
        }

        return false;
    }
    
    private void iterateBuddhabrot(double re, double im) {
        double x = 0;
        double y = 0;

        //System.out.println(re + " , " + im);
        
        for (int i = 0 ; i < maxIter ; i++) {
            double nx = x * x - y * y;
            double ny = 2 * x * y;

            x = nx + re;
            y = ny + im;
            if ((x == Double.NaN) || (y == Double.NaN) || (x == Double.POSITIVE_INFINITY) || (y == Double.POSITIVE_INFINITY) || (x == Double.NEGATIVE_INFINITY) || (y == Double.NEGATIVE_INFINITY)) {
                return;
            }
            if ((x * x + y * y) > 100) {
                return;
            }

            if ((x >= minX) && (x < maxX) && (y >= minY) && (y < maxY)) {
                int px = (int) ((x - minX) / pixelWidth);
                int py = (int) ((y - minY) / pixelWidth);

                //    System.out.println(x  + " --- " + y);
                //    System.out.println(px  + " +++ " + py);
                //  if ((px >= 0) && (px < pw) && (py >= 0) && (py < ph)) {
                image[px][py]++;

            //    py = ph - 1 - py;  // Simetrico
                //    image[px][py]++;
                //  }
            }

        }
    }

    


}
