/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package buddhabrot;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import javax.imageio.ImageIO;
import me.lemire.integercompression.Composition;
import me.lemire.integercompression.FastPFOR;
import me.lemire.integercompression.IntWrapper;
import me.lemire.integercompression.IntegerCODEC;
import me.lemire.integercompression.VariableByte;

/**
 *
 * @author zerjillo
 */
public class Utils {

    public static int[][] loadImageAsArray(String fileName) throws IOException {
        int[][] image = null;

        try {
            ObjectInputStream inputStream = new ObjectInputStream(new FileInputStream(fileName + ".iraw"));
            image = (int[][]) inputStream.readObject();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            System.exit(-1);
        }

        return image;
    }

    public static void delay(long millis) {
        try {
            Thread.sleep(millis);
        } catch (InterruptedException e) {
        }
    }

    public static void saveImageAsArray(String fileName, int[][] image) {
        try {
            ObjectOutputStream outputStream = new ObjectOutputStream(new FileOutputStream(fileName + ".iraw"));
            outputStream.writeObject(image);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void guardaImagen(String fileName, String format, int[][] image) {
        BufferedImage fractal = createImage(image);

        try {
            ImageIO.write(fractal, format, new File(fileName + "." + format));
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    // Creates a grayscale image from an array of ints.  0=black, 255=white.
    private static BufferedImage createImage(int image[][]) {

        int width = image.length;
        int height = image[0].length;

        int max = getMax(image);
        System.out.println("MAX: " + max);

        // Convert the 2D data into a 1D array:
        int oneD[] = new int[width * height];
        // For each row
        for (int i = 0 ; i < width ; i++) {
            // Check if this row has the correct number of pixels
            if (image[i].length != height) {
                System.out.println("Image is not rectangular");
                return null;
            }

            // for each column
            for (int j = 0 ; j < height ; j++) {
                int pixel = image[i][j];
                //   pixel = (int) (((double) pixel) / ((double) max) * 255);
                pixel = (int) ((Math.sqrt((double) pixel)) / Math.sqrt(((double) max)) * 255);
                // check for invalid values
                if (pixel > 255) {
                    pixel = 255;
                    System.err.println("*");
                } else if (pixel < 0) {
                    pixel = 0;
                }

                // Combine each pixel into one int.
                // First 8 bits are alpha value (255=opaque)
                // Then red, green, and blue (8-bits each)
                oneD[j * width + i] = 0xFF000000
                        | (0x000000FF & pixel) << 16
                        | (0x000000FF & pixel) << 8
                        | (0x000000FF & pixel);
            }
        }

        BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        img.setRGB(0, 0, width, height, oneD, 0, width);
        return img;
    }

    public static int getMax(int[][] image) {
        /*   int max = 0;
         int width = image.length;
         int height = image[0].length;

         for (int x = 0 ; x < width ; x++) {
         for (int y = 0 ; y < height ; y++) {
         if (image[x][y] > max) {
         max = image[x][y];
         }
         }
         }

         return max;
         */
        return getMax(image, 0, 0, image.length - 1, image[0].length - 1);
    }

    // Get the maximum between the coordinates (x1, y1) - (x2, y2) (included)
    public static int getMax(int[][] image, int x1, int y1, int x2, int y2) {
        int max = 0;

        for (int x = x1 ; x <= x2 ; x++) {
            for (int y = y1 ; y <= y2 ; y++) {
                if (image[x][y] > max) {
                    max = image[x][y];
                }
            }
        }

        return max;
    }

    public static long computeTotalCount(int[][] image) {
        int width = image.length;
        int height = image[0].length;

        long totalCount = 0;
        for (int x = 0 ; x < width ; x++) {
            for (int y = 0 ; y < height ; y++) {
                totalCount += image[x][y];
            }
        }

        return totalCount;
    }

    static int cuenta = 0;
    static int size = 0;

    public static void divideRecursivo(int[][] image, int x1, int y1, int x2, int y2, int maxWidth) {

        if (x2 - x1 < maxWidth) {
            System.out.println("[" + x1 + ", " + y1 + "] - [" + x2 + ", " + y2 + "]");
            cuenta++;
            int max = getMax(image, x1, y1, x2, y2);

            System.out.println(cuenta + " -> " + max + " ---- " + ((x2 - x1) * (y2 - y1)));

            int[] data = getPartImageAsArray(image, x1, y1, x2, y2);

            int[] compressed = new int[data.length + 1024];// could need more
            IntegerCODEC codec = new Composition(
                    new FastPFOR(),
                    new VariableByte());
            // compressing
            IntWrapper inputoffset = new IntWrapper(0);
            IntWrapper outputoffset = new IntWrapper(0);
            codec.compress(data, inputoffset, data.length, compressed, outputoffset);
            //System.out.println("compressed unsorted integers from "+data.length*4/1024+"KB to "+outputoffset.intValue()*4/1024+"KB");
            // we can repack the data: (optional)
            compressed = Arrays.copyOf(compressed, outputoffset.intValue());

            System.out.println(data.length + " + + " + compressed.length + " + + ");

            size += compressed.length;

            System.out.println(size);
            return;
        }

        int xMedio = x1 + (x2 - x1) / 2;
        int yMedio = y1 + (y2 - y1) / 2;

        divideRecursivo(image, x1, y1, xMedio, yMedio, maxWidth);
        divideRecursivo(image, x1, yMedio + 1, xMedio, y2, maxWidth);
        divideRecursivo(image, xMedio + 1, y1, x2, yMedio, maxWidth);
        divideRecursivo(image, xMedio + 1, yMedio + 1, x2, y2, maxWidth);
    }

    private static int[] getPartImageAsArray(int[][] image, int x1, int y1, int x2, int y2) {
        int w = x2 - x1 + 1;
        int h = y2 - y1 + 1;

        int[] res = new int[w * h];

        for (int x = x1 ; x <= x2 ; x++) {
            for (int y = y1 ; y <= y2 ; y++) {
                res[x - x1 + (y - y1) * w] = image[x][y];
            }
        }
        return res;
    }

    private static void setPartImageFromArray(int[][] image, int x1, int y1, int x2, int y2, int[] array) {
        int w = x2 - x1 + 1;
        int h = y2 - y1 + 1;

        for (int x = x1 ; x <= x2 ; x++) {
            for (int y = y1 ; y <= y2 ; y++) {
                image[x][y] = array[x - x1 + (y - y1) * w];
            }
        }
    }

    public static void saveCompressed(String fileName, int[][] image) throws Exception {
        int w = image.length;
        int h = image[0].length;


            ObjectOutputStream outputStream = new ObjectOutputStream(new FileOutputStream(fileName + ".ciraw"));
            outputStream.writeObject(w);
            outputStream.writeObject(h);

            for (int i = 0 ; i < image.length ; i++) {
                int[] data = image[i];
                int[] compressed = new int[data.length + 1024];// could need more
                IntegerCODEC codec = new Composition(
                        new FastPFOR(),
                        new VariableByte());
                // compressing
                IntWrapper inputoffset = new IntWrapper(0);
                IntWrapper outputoffset = new IntWrapper(0);
                codec.compress(data, inputoffset, data.length, compressed, outputoffset);
                //System.out.println("compressed unsorted integers from "+data.length*4/1024+"KB to "+outputoffset.intValue()*4/1024+"KB");
                // we can repack the data: (optional)
                compressed = Arrays.copyOf(compressed, outputoffset.intValue());

                outputStream.writeObject(compressed);


            }


            outputStream.close();


    }

    public static int[][] loadCompressed(String fileName) throws Exception {
        int[][] image = null;

            ObjectInputStream inputStream = new ObjectInputStream(new FileInputStream(fileName + ".ciraw"));

            int w = (Integer) inputStream.readObject();
            int h = (Integer) inputStream.readObject();

            image = new int[w][];

            for (int i = 0 ; i < w ; i++) {
                int[] compressed = (int[]) inputStream.readObject();

                int[] recovered = new int[h];
                IntWrapper recoffset = new IntWrapper(0);
                IntegerCODEC codec = new Composition(
                        new FastPFOR(),
                        new VariableByte());
                codec.uncompress(compressed, new IntWrapper(0), compressed.length, recovered, recoffset);
                image[i] = recovered;
            }


        return image;
    }
}
